#!/usr/bin/wish

# GT - GUI wrapper for the Genesis Toolkit
#
# Genesis Toolkit is (C) 1998-2002
# http://www.schrammel.org/
# Object placement for POV-Ray terrains
#
# GT is (C) 2002
# http://www.bowlvisions.com/
# GUI for Genesis, POV-Ray lights & camera
#
# ithaqua@inreach.com
#
# TBD - runtime assignment of scale -from and -to is not allowed
# according to experiments... so maybe add 100 1000 radio buttons.
# TBD - save all GT parameters on exit, restore on entry.
# TBD - arrays of variables, a few generic procedures.
# TBD - file action error handlers - currently captured by the
#        Tcl/Tk v8.3.4 interpreters.
#
#################
set version v0.09
# Cutter done, client starvation issue is now cleared.
#
#################
# set version v0.08
# Cutter panel now has Object scale controls.
# Cutter now produces a display.
#
# TBD - Cutter output display working correctly?  Test on flat terrain.
#
#################
# set version v0.07
#
# Mode-lock active, Gen XYZ scaling ONLY allowed during:
# Camera setting OR Light setting, to force write-updates
# to camera.inc and camera.pov files upon scale changes
# (implicit synch now enabled).
#
# New default values for cool included El Dorado terrain.
#
# TBD - Cutter operation output display...
#
#################
# set version v0.06
# Lights now track scale, like Camera did already.
# Mode-lock for Gen X, Z, and Y is not working, so explicit
# synch by activating Camera or Lights is still required.
#
##################
#set version v0.05
# Multiple stacked control-panel frames selected by tabs.
# Common canvas for all apps.
#
# Selector buttons choose:
# a) Applications control panels for Rough, Gen and Cutter.
# Also, you can double-click Rough, Gen and Cutter selector buttons to
# immediately activate Rough, Gen and Cutter execution.
# You get easy data propogation for "just-one-tweak" edits.
#
# or
# b) Settings panels for Lights and Camera.
# Fun to play with, modal for easy undo, updates camera.inc and camera.pov.
# Cutter uses the camera.pov file.
# Note that camera-set and light-set operations are modal,
# so that the file will be written only after confirmation...
# This makes undo with repositioning easy.
#
# or
# c) Object info and statistics (Objs) - TBD, Help - also TBD
# Objs and Help buttons do nothing.
#
###################
# set version v0.04
# Made canvas widget for GEN object overlay.  GIF representation from
# ROUGH app is displayed, then overlayed with GENed objects.
#
###################
# set version v0.03
# GEN is enabled and working fine.
# All ROUGH and GEN controls are finished except hardwired X, Y, and Z
# in GEN.  Frames stripped, grid layout is okay, but not pretty.
#
###################
# set version v0.02
# Only ROUGH is enabled so far.
# Finish controls.  Strip frames, try everything-in-a-grid layout.
#
###################
# set version v0.01
# Initial hack to get everything working and lay out good presentation.
# Very easy-to-debug structure... each trivial procedure is explicit,
# even though a few generic procedures would suffice.
# ROUGH is working fine, with hardwired filenames.
# ROUGH, GEN, CUTTER, HFMERGE, GIF2TGA, TGA2GIF

#

wm title . "GT $version for Genesis Toolkit"

# --- theme, layout, color scheme ---

# theme style
#
# relFrame              relief type for frames
# spFrame               space around frames
# xFrame,yFrame         frame dimensions
#
# widthGIF,heightGIF    pixel dimensions of GIF
# xlocGIF,ylocGIF       center of the GIF
# widthText               text entry box width
#
set relFrame   groove
set spFrame      2
set xFrame     200
set yFrame     500
set heightKeys   1
set widthText   27
set widthBut     3

# ints needed for graphics locations
set widthGIF   500
set heightGIF  500
#set xlocGIF    [ expr ($widthGIF / 2 ) ]
#set ylocGIF    [ expr ($heightGIF / 2 ) ]
set xlocGIF    [ expr ($widthGIF / 2 ) + 3 ]
set ylocGIF    [ expr ($heightGIF / 2 ) + 3 ]

# floats needed for calculating graphics
set widthGIFf   500.00
set heightGIFf  500.00

# camera, light, objects ... graphical display definitions
#
set widthLine    2
set treeRadius   2.5

# dimensions of scale widgets
#
set lengthGenXYZ 160
# set lengthGenXYZ 140
set lengthZY   96
set widthScale 10

# theme colors
# action buttons are colored rgbxxxB  (for Brighter)
# scale troughs  are colored rgbxxxDk (for Darker)
#
set colorText     #ffffff
set rgbCutter     #ffcc66

set rgbGenB       #bbeeee
set rgbGen        #99cccc
set rgbGenDk      #669999

set rgbFileOp     #ffeebb
set rgbFileOpB    #ffffcc
set rgbFileOpDk   #cc9966

set rgbRoughB     #ccffcc
set rgbRough      #99cc99
set rgbRoughDk    #669966

set rgbGenB       #bbeeee
set rgbGen        #99cccc
set rgbGenDk      #669999

set rgbCamera     #ff0000
set rgbCameraLt   #ff9999
set rgbCameraDk   #cc6666

set rgbLight      #0000ff
set rgbLightLt    #77aadd
set rgbLightDk    #4477aa

set rgbCutterB    #ffbb99
set rgbCutter     #ff9977
set rgbCutterDk   #dd7755

# --- color ref ---
#
   # manila ffeebb
   # saffron ffffcc
   # brass ffd066
   # goldenrod cccc99
   # orange template ffcc99

   # near forest green-yellow 339933
   # softer near forest green 40c080
   # green LED on 66ff99
   # nice old template green 70c0b8
   # bright drafting table green ccffcc
   # taffy green 99cc99

   # nice bluegrey b4b8bf
   # robin's egg blue bbeeee
   # soft slate blue 99cccc

   # periwinkleLt 99bbff   aabbff
   # periwinkle 88aaee     99aaee
   # periwinkleDk 7799dd   8899dd

   # violetLt bbaaee
   # violet aa99dd
   # violetDk 9988cc

   # lavenderLt ffccff
   # lavender eeaaee
   # lavenderDk dd88dd

# Sign-on image
#
set imageRough [image create photo -format gif -width $widthGIF -height $heightGIF -file gtsign.gif]


# Help, setCamera , setLight, objectStats
# global widthBut is already used
#
set widthButK 15

# scaling from POV
#
set scaleWidth    32.00
set scaleLength   $scaleWidth
set scaleHeight   [expr $scaleWidth / 8]

# cameraRange * scaleHeight is max height for camera
# lightRange  * scaleHeight is max height for lights
#
set cameraRange   4
set lightRange    4

# camera settings from POV
#
set cameraX       [expr ( $scaleWidth  /  4 ) *  3.0]
set cameraZ       [expr ( $scaleLength /  4 ) *  3.0]
set cameraY       [expr ( $scaleHeight / 16 ) * 11.0]
set cameraLookX   [expr ( $scaleWidth  /  4 ) *  3.0]
set cameraLookZ   0.0
set cameraLookY   [expr ( $scaleHeight /  4 ) *  3.5]

# initial scale tracking normalization for camera
# Normalizations are used during Gen scaling adjustments to
# make the camera position track terrain scale in realtime
#
set cameraXn      [expr $cameraX       / $scaleWidth  ]
set cameraZn      [expr $cameraZ       / $scaleLength ]
set cameraYn      [expr $cameraY       / $scaleHeight ]
set cameraLookXn  [expr $cameraLookX   / $scaleWidth  ]
set cameraLookZn  [expr $cameraLookZ   / $scaleLength ]
set cameraLookYn  [expr $cameraLookY   / $scaleHeight ]

# light settings from POV
#
set light1x    [expr ( $scaleWidth  / 8 ) *  4.5]
set light1z    [expr ( $scaleLength / 8 ) *  2.0]
set light1y    [expr ( $scaleHeight / 8 ) * 30.0]

set light2x    [expr ( $scaleWidth  / 8 ) *  5.0]
set light2z    [expr ( $scaleLength / 8 ) *  5.0]
set light2y    [expr ( $scaleHeight / 8 ) * 16.0]

set light3x    [expr ( $scaleWidth  / 4 ) *  3.0]
set light3z    [expr ( $scaleLength / 4 ) *  3.0]
set light3y    [expr ( $scaleHeight / 4 ) *  3.0]

# initial scale tracking normalization for lights
# Normalizations are used during Gen scaling adjustments to
# make the light positions track terrain scale in realtime
#
set light1xn   [expr $light1x / $scaleWidth  ]
set light1zn   [expr $light1z / $scaleLength ]
set light1yn   [expr $light1y / $scaleHeight ]

set light2xn   [expr $light2x / $scaleWidth  ]
set light2zn   [expr $light2z / $scaleLength ]
set light2yn   [expr $light2y / $scaleHeight ]

set light3xn   [expr $light3x / $scaleWidth  ]
set light3zn   [expr $light3z / $scaleLength ]
set light3yn   [expr $light3y / $scaleHeight ]

# internal - size of displayed light, max height settings for camera and lights
set radiusLight 5
set maxHeightLight   [expr $scaleHeight * 8]
set maxHeightCamera  [expr $scaleHeight * 8]

# Modes, completion...
# Rough done, Gen done, camera.inc written
set doneRough  0
set doneGen    0
set doneInc    0

# modal window active
# use "active", "disabled", or "normal"
# to match option called -state
#
set modalActive "disabled"

set fileCameraInc "camera.inc"
set bufferCameraInc " --- init for file data --- "

set fileCameraPov "camera.pov"
set bufferCameraPov " --- init for file data --- "

# --- Implicit file ops for camera and light

# Refresh the data required by dataWriteInc.
#
# The file camera.inc is really built here, the GT input data file parser
# expects this format.  camera.inc is POV-Ray script, the symbolic declarations
# for Camera, Light and Scale.
#
# Edit the values in camera.inc outside GT if you like, but don't hack it up.
# Meaning especially:
# Do NOT change the order of objects in camera.inc or else,
# communication between GT and POV-Ray will be lost.
#
# The file camera.pov is also built here.
# This file has two uses,
# 1. Cutter uses literal camera definitions from file: camera.pov,
# built here.
# 2. camera.pov data may be used as-is in a POV-Ray scene that does not
# use symbolic declarations from camera.inc.
#
proc refreshInc {} {
   global scaleWidth    scaleLength    scaleHeight
   global cameraX       cameraZ        cameraY
   global cameraLookX   cameraLookZ    cameraLookY
   global light1x       light1z        light1y
   global light2x       light2z        light2y
   global light3x       light3z        light3y

   global bufferCameraInc
   global bufferCameraPov

   puts stdout "GT refresh camera.inc data buffer "
   set bufferCameraInc "// camera.inc include file for POV-Ray \t\
\n// GT-generated file for scale, camera, and lights.          \t\
\n// GT is a GUI for the Genesis Toolkit.                      \t\
\n\t\
\n#declare scaleWidth = $scaleWidth;   \t\
\n#declare scaleLength = $scaleLength; \t\
\n#declare scaleHeight = $scaleHeight; \t\
\n\t\
\n#declare cameraX = $cameraX;   \t\
\n#declare cameraZ = $cameraZ;   \t\
\n#declare cameraY = $cameraY;   \t\
\n\t\
\n#declare cameraLookX = $cameraLookX; \t\
\n#declare cameraLookZ = $cameraLookZ; \t\
\n#declare cameraLookY = $cameraLookY; \t\
\n\t\
\n#declare light1x = $light1x;   \t\
\n#declare light1z = $light1z;   \t\
\n#declare light1y = $light1y;   \t\
\n\t\
\n#declare light2x = $light2x;   \t\
\n#declare light2z = $light2z;   \t\
\n#declare light2y = $light2y;   \t\
\n\t\
\n#declare light3x = $light3x;   \t\
\n#declare light3z = $light3z;   \t\
\n#declare light3y = $light3y;   \t\
\n\t\
\n// end of camera.inc, GT-generated file \n"

   puts stdout "GT refresh camera.pov data buffer "
   set bufferCameraPov "// camera.pov literal camera file for POV-Ray   \t\
\n// GT-generated file for POV-Ray camera.   \t\
\n// GT is a GUI for the Genesis Toolkit.    \t\
\n// \t\
\n\t\
\ncamera {                                  \t\
\n   location<$cameraX, $cameraY, $cameraZ> \t\
\n   direction<0,0,1>                       \t\
\n   up<0,1,0>                              \t\
\n   right<1.3333333,0,0>                   \t\
\n   look_at<$cameraLookX, $cameraLookY, $cameraLookZ> \t\
\n}   \t\
\n\t\
\n// end of camera.pov, GT-generated file \n"
}

# dataWriteInc
# Write POV-Ray script camera.inc,
#
proc dataWriteInc {} {
   global fileCameraInc
   global bufferCameraInc
   global fileCameraPov
   global bufferCameraPov

   # open camera.inc file for write
   set f [ open $fileCameraInc w ]

   # write camera.inc file
   puts stdout "GT write $fileCameraInc data buffer "
   puts $f $bufferCameraInc

   # close camera.inc file (causes flush too), after write
   close $f
   puts stdout "GT $fileCameraInc written "

   # open camera.pov file for write
   set f [ open $fileCameraPov w ]

   # write camera.pov file
   puts stdout "GT writing $fileCameraPov data buffer "
   puts $f $bufferCameraPov

   # close camera.pov file (causes flush too), after write
   close $f
   puts stdout "GT $fileCameraPov written "
}

# Read POV-Ray script camera.inc, made by dataWriteInc
#
proc dataReadInc {} {
   global scaleWidth    scaleLength    scaleHeight

   global cameraX       cameraZ        cameraY
   global cameraXn      cameraZn       cameraYn
   global cameraLookX   cameraLookZ    cameraLookY
   global cameraLookXn  cameraLookZn   cameraLookYn

   global light1x       light1z        light1y
   global light1xn      light1zn       light1yn
   global light2x       light2z        light2y
   global light2xn      light2zn       light2yn
   global light3x       light3z        light3y
   global light3xn      light3zn       light3yn

   global fileCameraInc

   set dataVarInc {  scaleWidth     scaleLength    scaleHeight \
                     cameraX        cameraZ        cameraY     \
                     cameraLookX    cameraLookZ    cameraLookY \
                     light1x        light1z        light1y     \
                     light2x        light2z        light2y     \
                     light3x        light3z        light3y     }
   set iVar 0

   while { $iVar < 18 } {

      # var is now assigned the string name of a variable like scaleWidth
      set var [ lindex $dataVarInc $iVar ]

      # display the string name of the current variable chosen by $iVar
      # puts $var
      # puts " \n"

      # assign to the selected variable a new value
      set $var $iVar

      # prep to iterate the loop
      set iVar [ expr $iVar + 1 ]
   }

   # display all variable data
   # puts "$scaleWidth    $scaleLength   $scaleHeight   \n"
   # puts "$cameraX       $cameraZ       $cameraY       \n"
   # puts "$cameraLookX   $cameraLookZ   $cameraLookY   \n"
   # puts "$light1x       $light1z       $light1y       \n"
   # puts "$light2x       $light2z       $light2y       \n"
   # puts "$light3x       $light3z       $light3y       \n"

   # ACHTUNG: initialize variables to junk float values (in this case),
   # OR ELSE expr will not understand them - even after subsequent
   # assignments that appear okay EVEN using puts (because you see a string)!
   # Failure to initialize caused total code failures...
   # .. with mystery error reports - an added BONUS.

   # var is now assigned the string name of a variable like scaleWidth
   set iVar 0
   set var [ lindex $dataVarInc $iVar ]

   # open file for read
   set f [ open $fileCameraInc ]

   # locate and assign each variable, from file data
   while { $iVar < 18 } {
      set var [ lindex $dataVarInc $iVar ]
      while { [ gets $f dataCam ] != -1 } {

         # find and display string from list
         set t [string first $var $dataCam]

         if { $t == -1 } { puts -nonewline "not found, "  } \
         else {
            # clean string, remove special chars
            regsub -all --  ";"  $dataCam " " dataCamCopy
            regsub -all --  "#"  $dataCamCopy " " dataCam

            # display string index to first character of specified string
            puts "\n $t "

            # adjust to space after = sign
            set t [expr $t + [string length $var] + 2]

            # display value of variable
            puts -nonewline "$t "

            # get numerical substring, assign to
            set numdataCam [string range $dataCam $t end]

            # preset appropriate variable
            set valFP   666.66
            # assign to appropriate variable
            scan $numdataCam "%f" valFP

            # display - again
            puts -nonewline "$dataCam "
            puts -nonewline "$valFP \n"

            # assign value valFP to the variable selected as var, then break to var loop
            set $var $valFP
            break
         }
      }
      set iVar [ expr $iVar + 1 ]
   }

   # close file after read
   close $f

   #  initialize normalizations for scaling
   set cameraXn      [expr $cameraX       / $scaleWidth  ]
   set cameraZn      [expr $cameraZ       / $scaleLength ]
   set cameraYn      [expr $cameraY       / $scaleHeight ]

   set cameraLookXn  [expr $cameraLookX   / $scaleWidth  ]
   set cameraLookZn  [expr $cameraLookZ   / $scaleLength ]
   set cameraLookYn  [expr $cameraLookY   / $scaleHeight ]

   set light1xn      [expr $light1x       / $scaleWidth  ]
   set light1zn      [expr $light1z       / $scaleLength ]
   set light1yn      [expr $light1y       / $scaleHeight ]

   set light2xn      [expr $light2x       / $scaleWidth  ]
   set light2zn      [expr $light2z       / $scaleLength ]
   set light2yn      [expr $light2y       / $scaleHeight ]

   set light3xn      [expr $light3x       / $scaleWidth  ]
   set light3zn      [expr $light3z       / $scaleLength ]
   set light3yn      [expr $light3y       / $scaleHeight ]
}

####################################################################
# Immediate initialization for camera, lights and scale data objects
#
dataReadInc

# --- Camera ---

# Live camera display upon adjustment,
# Camera is also redrawn at the end of Gen
#
set cameraLock 1

variable oldCameraX
variable oldCameraZ
variable oldCameraY
variable oldCameraLookX
variable oldCameraLookZ
variable oldCameraLookY

set oldCameraX       $cameraX
set oldCameraZ       $cameraZ
set oldCameraY       $cameraY
set oldCameraLookX   $cameraLookX
set oldCameraLookZ   $cameraLookZ
set oldCameraLookY   $cameraLookY

variable oldCamera

proc initCamera {} {
   global modalActive
   global oldCameraX       oldCameraZ        oldCameraY
   global oldCameraLookX   oldCameraLookZ    oldCameraLookY
   global cameraX          cameraZ           cameraY
   global cameraLookX      cameraLookZ       cameraLookY

   set modalActive "active"

   # TBD - repack does not activate new -state option "active"
   #
   # pack .fMisc.fCtrl.fGenXYZ.sGenX
   # pack .fMisc.fCtrl.fGenXYZ.sGenZ
   # pack .fMisc.fCtrl.fGenXYZ.sGenY

   set oldCameraX       $cameraX
   set oldCameraZ       $cameraZ
   set oldCameraY       $cameraY
   set oldCameraLookX   $cameraLookX
   set oldCameraLookZ   $cameraLookZ
   set oldCameraLookY   $cameraLookY
}
proc drawCamera {} {
   global widthLine
   global widthGIFf heightGIFf
   global scaleWidth scaleLength
   global cameraX  cameraZ
   global cameraLookX cameraLookZ
   global oldCamera

   set xLo [expr               (( $widthGIFf  / $scaleWidth  ) * $cameraX)]
   set zLo [expr $heightGIFf - (( $heightGIFf / $scaleLength ) * $cameraZ)]
   set xHi [expr               (( $widthGIFf  / $scaleWidth  ) * $cameraLookX)]
   set zHi [expr $heightGIFf - (( $heightGIFf / $scaleLength ) * $cameraLookZ)]

   .fMisc.cGraphic delete $oldCamera
   set oldCamera [ .fMisc.cGraphic create line $xLo $zLo $xHi $zHi -arrow last -fill Red -width $widthLine]
}
proc ctrlCameraUndo {} {
   global scaleWidth       scaleLength       scaleHeight

   global oldCameraX       oldCameraZ        oldCameraY
   global cameraX          cameraZ           cameraY
   global cameraXn         cameraZn          cameraYn

   global oldCameraLookX   oldCameraLookZ    oldCameraLookY
   global cameraLookX      cameraLookZ       cameraLookY
   global cameraLookXn     cameraLookZn      cameraLookYn

   # restore old camera positions
   set cameraX       $oldCameraX
   set cameraZ       $oldCameraZ
   set cameraY       $oldCameraY
   set cameraLookX   $oldCameraLookX
   set cameraLookZ   $oldCameraLookZ
   set cameraLookY   $oldCameraLookY

   # restore old camera normalizations
   set cameraXn      [expr $cameraX / $scaleWidth]
   set cameraZn      [expr $cameraZ / $scaleLength]
   set cameraYn      [expr $cameraY / $scaleHeight]
   set cameraXLookn  [expr $cameraLookX / $scaleWidth]
   set cameraZLookn  [expr $cameraLookZ / $scaleLength]
   set cameraYLookn  [expr $cameraLookY / $scaleHeight]

   drawCamera
}
proc ctrlCameraDone {} {
   global modalActive
   global app        oldApp
   global colorApp   oldColorApp
   global nameApp    oldNameApp

   # refresh output data with new variables
   refreshInc
   # write camera, lights and scale to camera.inc
   dataWriteInc

   grid remove .f$app
   set app        $oldApp
   set colorApp   $oldColorApp
   set nameApp    $oldNameApp
   grid .f$app -column 0 -row 0 -sticky "sw"
   set modalActive "disabled"
}

# TBD - check y axes, limit-to-max rule
# Normalizations are done here instead of drawCamera, because
# drawCamera is not (yet) used for y axis updates
#
proc setCameraX {x} {
   global cameraX cameraXn scaleWidth

   # limit camera position to width of terrain
   if { $x > $scaleWidth } { set cameraX $scaleWidth } else { set cameraX $x }
   # adjust camera normalization to scale
   set cameraXn [expr $cameraX / $scaleWidth]
   drawCamera
}
proc setCameraZ {z} {
   global cameraZ cameraZn scaleLength

   # limit camera position to length of terrain
   if { $z > $scaleLength } { set cameraZ $scaleLength } else { set cameraZ $z }
   # adjust camera normalization to scale
   set cameraZn [expr $cameraZ / $scaleLength]
   drawCamera
}
proc setCameraY {y} {
   global cameraY cameraYn scaleHeight cameraRange

   # limit-to-max rule
   if { $y > [expr $scaleHeight * $cameraRange] } { set cameraY [expr $scaleHeight * $cameraRange] } else { set cameraY $y }
   # adjust camera normalization to scale
   set cameraYn [expr $cameraY / $scaleHeight]
}
proc setCameraLookX {x} {
   global cameraLookX cameraLookXn scaleWidth

   # limit target position to width of terrain
   if { $x > $scaleWidth } { set cameraLookX $scaleWidth } else { set cameraLookX $x }
   set cameraLookXn [expr $cameraLookX / $scaleWidth]
   drawCamera
}
proc setCameraLookZ {z} {
   global cameraLookZ cameraLookZn scaleLength

   # limit target position to length of terrain
   if { $z > $scaleLength } { set cameraLookZ $scaleLength } else { set cameraLookZ $z }
   set cameraLookZn [expr $cameraLookZ / $scaleLength]
   drawCamera
}
proc setCameraLookY {y} {
   global cameraLookY cameraLookYn scaleHeight cameraRange

   # limit-to-max rule
   if { $y > [expr $scaleHeight * $cameraRange] } { set cameraLookY [expr $scaleHeight * $cameraRange] } else { set cameraLookY $y }
   set cameraLookYn [expr $cameraLookY / $scaleHeight]
}

# Camera controls
#
frame    .fCtrlCamera         -background $rgbCameraLt  -borderwidth $spFrame   \
                              -width $xFrame -height $yFrame -relief $relFrame

label    .fCtrlCamera.lFile   -background $rgbCameraLt  -text "Pov camera include: "
entry    .fCtrlCamera.eFile   -background $colorText  -textvariable fileCameraInc -width [expr $widthText - 1]

label    .fCtrlCamera.lCamPos -background $rgbCameraLt  -text "Camera positioning"
label    .fCtrlCamera.lX      -background $rgbCameraLt  -text "Camera x "
entry    .fCtrlCamera.eX      -background $colorText  -textvariable cameraX -width $widthText
label    .fCtrlCamera.lZ      -background $rgbCameraLt  -text "Camera z "
entry    .fCtrlCamera.eZ      -background $colorText  -textvariable cameraZ -width $widthText


label    .fCtrlCamera.lTargPos -background $rgbCameraLt  -text "Target position"
label    .fCtrlCamera.lLookX  -background $rgbCameraLt  -text "Look-at x "
entry    .fCtrlCamera.eLookX  -background $colorText  -textvariable cameraLookX -width $widthText
label    .fCtrlCamera.lLookZ  -background $rgbCameraLt  -text "Look-at z "
entry    .fCtrlCamera.eLookZ  -background $colorText  -textvariable cameraLookZ -width $widthText

scale    .fCtrlCamera.sCameraX   -background $rgbCameraLt \
                                 -command "setCameraX " -label "x left-right"  \
                                 -showvalue 1 -from   0 -to [expr $scaleWidth * 2] -resolution 0.10 \
                                 -orient horizontal -troughcolor $rgbCameraDk \
                                 -variable cameraX -width $widthScale

frame    .fCtrlCamera.fCameraZY  -background $rgbCameraLt
scale    .fCtrlCamera.fCameraZY.sCameraZ   -background $rgbCameraLt \
                                 -command "setCameraZ " -label "z"  \
                                 -showvalue 1 -from [expr $scaleLength * 2] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbCameraDk \
                                 -variable cameraZ -length $lengthZY -width $widthScale

scale    .fCtrlCamera.fCameraZY.sCameraY   -background $rgbCameraLt \
                                 -command "setCameraY " -label "y"  \
                                 -showvalue 1 -from [expr $scaleHeight * $cameraRange] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbCameraDk \
                                 -variable cameraY -length $lengthZY -width $widthScale

label    .fCtrlCamera.fCameraZY.lZY -background $rgbCameraLt \
               -text "Camera:\n\n  y\nlow-high\n  z\nnear-far" -justify left

pack configure \
   .fCtrlCamera.fCameraZY.lZY       \
   .fCtrlCamera.fCameraZY.sCameraY  \
   .fCtrlCamera.fCameraZY.sCameraZ  \
   -side left -fill x -anchor "w" -expand 0

scale    .fCtrlCamera.sCameraLookX   -background $rgbCameraLt \
                                 -command "setCameraLookX " -label "x left-right"  \
                                 -showvalue 1 -from   0 -to [expr $scaleWidth * 2] -resolution 0.10 \
                                 -orient horizontal -troughcolor $rgbCameraDk \
                                 -variable cameraLookX -width $widthScale

frame    .fCtrlCamera.fTargetZY  -background $rgbCameraLt
scale    .fCtrlCamera.fTargetZY.sCameraLookZ   -background $rgbCameraLt \
                                 -command "setCameraLookZ " -label "z"  \
                                 -showvalue 1 -from [expr $scaleLength * 2] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbCameraDk \
                                 -variable cameraLookZ -length $lengthZY -width $widthScale

scale    .fCtrlCamera.fTargetZY.sCameraLookY   -background $rgbCameraLt \
                                 -command "setCameraLookY " -label "y"  \
                                 -showvalue 1 -from [expr $scaleHeight * $cameraRange] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbCameraDk \
                                 -variable cameraLookY -length $lengthZY -width $widthScale

label    .fCtrlCamera.fTargetZY.lZY -background $rgbCameraLt   \
                     -text "Target:\n\n  y\nlow-high\n  z\nnear-far" -justify left

pack configure \
   .fCtrlCamera.fTargetZY.lZY          \
   .fCtrlCamera.fTargetZY.sCameraLookY \
   .fCtrlCamera.fTargetZY.sCameraLookZ \
   -side left -fill x -anchor "w" -expand 0

checkbutton .fCtrlCamera.cCameraLock -background $rgbCameraLt \
                           -text "lock" -variable cameraLock \
                           -anchor "w"
                           # -height 1

button   .fCtrlCamera.bUndo   -background $rgbCameraLt   -command "ctrlCameraUndo" -text "Undo "  \
                              -width $widthBut
button   .fCtrlCamera.bDone   -background $rgbCamera     -command "ctrlCameraDone"  -foreground White -text "Done "  \
                              -width $widthBut
button   .fCtrlCamera.bExit   -background $rgbCameraLt   -command exit -text "Exit "  \
                              -width $widthBut

label    .fCtrlCamera.lBlank0 -background $rgbCameraLt  -text " "
label    .fCtrlCamera.lBlank1 -background $rgbCameraLt  -text " "
label    .fCtrlCamera.lBlank2 -background $rgbCameraLt  -text " "
label    .fCtrlCamera.lBlank3 -background $rgbCameraLt  -text " "
label    .fCtrlCamera.lBlank4 -background $rgbCameraLt  -text " "
label    .fCtrlCamera.lBlank5 -background $rgbCameraLt  -text " "

label    .fCtrlCamera.lSpace0 -text " "
label    .fCtrlCamera.lSpace1 -text " "

pack configure \
   .fCtrlCamera.lFile      \
   .fCtrlCamera.eFile      \
   .fCtrlCamera.lBlank0    \
   .fCtrlCamera.fCameraZY  \
   .fCtrlCamera.sCameraX   \
   .fCtrlCamera.lBlank2    \
   .fCtrlCamera.fTargetZY  \
   .fCtrlCamera.sCameraLookX  \
   .fCtrlCamera.lBlank3    \
   .fCtrlCamera.bUndo      \
   .fCtrlCamera.bDone      \
   .fCtrlCamera.lBlank4    \
   -side top -fill x -anchor "nw" -expand 0

# --- Lights ---

# Live lights display upon adjustment,
# Lights also redrawn at the end of Gen

set lightLock 1

variable oldLight1
variable oldLight2
variable oldLight3

variable oldLight1x
variable oldLight1z
variable oldLight1y
variable oldLight2x
variable oldLight2z
variable oldLight2y
variable oldLight3x
variable oldLight3z
variable oldLight3y

set oldLight1x    $light1x
set oldLight1z    $light1z
set oldLight1y    $light1y
set oldLight2x    $light2x
set oldLight2z    $light2z
set oldLight2y    $light2y
set oldLight3x    $light3x
set oldLight3z    $light3z
set oldLight3y    $light3y

proc initLights {} {
   global modalActive
   global oldLight1x    oldLight1z     oldLight1y
   global oldLight2x    oldLight2z     oldLight2y
   global oldLight3x    oldLight3z     oldLight3y
   global light1x       light1z        light1y
   global light2x       light2z        light2y
   global light3x       light3z        light3y

   set modalActive "active"
   set oldLight1x    $light1x
   set oldLight1z    $light1z
   set oldLight2x    $light2x
   set oldLight2z    $light2z
   set oldLight3x    $light3x
   set oldLight3z    $light3z
}
proc drawLights {} {
   global widthGIFf  scaleWidth
   global heightGIFf scaleLength
   global radiusLight
   global light1x light1z  light2x light2z   light3x light3z
   global oldLight1        oldLight2         oldLight3

   set xLo [expr              (( $widthGIFf  / $scaleWidth  ) * $light1x) - $radiusLight]
   set zLo [expr $heightGIFf - (( $heightGIFf / $scaleLength ) * $light1z) - $radiusLight]
   set xHi [expr              (( $widthGIFf  / $scaleWidth  ) * $light1x) + $radiusLight]
   set zHi [expr $heightGIFf - (( $heightGIFf / $scaleLength ) * $light1z) + $radiusLight]

   .fMisc.cGraphic delete $oldLight1
   set oldLight1 [ .fMisc.cGraphic create oval $xLo $zLo $xHi $zHi -fill Blue ]

   set xLo [expr              (( $widthGIFf  / $scaleWidth  ) * $light2x) - $radiusLight]
   set zLo [expr $heightGIFf - (( $heightGIFf / $scaleLength ) * $light2z) - $radiusLight]
   set xHi [expr              (( $widthGIFf  / $scaleWidth  ) * $light2x) + $radiusLight]
   set zHi [expr $heightGIFf - (( $heightGIFf / $scaleLength ) * $light2z) + $radiusLight]

   .fMisc.cGraphic delete $oldLight2
   set oldLight2 [ .fMisc.cGraphic create oval $xLo $zLo $xHi $zHi -fill Blue ]

   set xLo [expr              (( $widthGIFf  / $scaleWidth  ) * $light3x) - $radiusLight]
   set zLo [expr $heightGIFf - (( $heightGIFf / $scaleLength ) * $light3z) - $radiusLight]
   set xHi [expr              (( $widthGIFf  / $scaleWidth  ) * $light3x) + $radiusLight]
   set zHi [expr $heightGIFf - (( $heightGIFf / $scaleLength ) * $light3z) + $radiusLight]

   .fMisc.cGraphic delete $oldLight3
   set oldLight3 [ .fMisc.cGraphic create oval $xLo $zLo $xHi $zHi -fill Blue ]
}
proc ctrlLightUndo {} {
   global oldLight1x    oldLight1z     oldLight1y
   global oldLight2x    oldLight2z     oldLight2y
   global oldLight3x    oldLight3z     oldLight3y

   global light1x       light1z        light1y
   global light2x       light2z        light2y
   global light3x       light3z        light3y

   global light1xn      light1zn       light1yn
   global light2xn      light2zn       light2yn
   global light3xn      light3zn       light3yn

   set light1x       $oldLight1x
   set light1z       $oldLight1z
   set light1y       $oldLight1y

   set light2x       $oldLight2x
   set light2z       $oldLight2z
   set light2y       $oldLight2y

   set light3x       $oldLight3x
   set light3z       $oldLight3z
   set light3y       $oldLight3y

   # restore old light normalizations
   set light1xn      [ expr $light1x / $scaleWidth  ]
   set light1zn      [ expr $light1z / $scaleLength ]
   set light1yn      [ expr $light1y / $scaleHeight ]

   set light2xn      [ expr $light2x / $scaleWidth  ]
   set light2zn      [ expr $light2z / $scaleLength ]
   set light2yn      [ expr $light2y / $scaleHeight ]

   set light3xn      [ expr $light3x / $scaleWidth  ]
   set light3zn      [ expr $light3z / $scaleLength ]
   set light3yn      [ expr $light3y / $scaleHeight ]

   drawLights
}
proc ctrlLightDone {} {
   global modalActive
   global app        oldApp
   global colorApp   oldColorApp
   global nameApp    oldNameApp

   # refresh output data with new variables
   refreshInc
   # write camera, lights and scale to camera.inc
   dataWriteInc

   grid remove .f$app
   set app        $oldApp
   set colorApp   $oldColorApp
   set nameApp    $oldNameApp
   grid .f$app -column 0 -row 0 -sticky "sw"
   set modalActive "disabled"
}

# TBD - check y axes, limit-to-max rule
# Normalizations are done here instead of drawLights, because
# drawLights is not (yet) used for y axis updates
#
proc setLight1x {x} {
   global light1x scaleWidth

   # limit light position to width of terrain
   if { $x > $scaleWidth } { set light1x $scaleWidth } else { set light1x $x }
   drawLights
}
proc setLight1z {z} {
   global light1z scaleLength

   # limit light position to length of terrain
   if { $z > $scaleLength } { set light1z $scaleLength } else { set light1z $z }
   drawLights
}
proc setLight1y {y} {
   global light1y light1yn lightRange scaleHeight

   # limit-to-max rule
   if { $y > [expr $scaleHeight * $lightRange] } { set light1y [expr $scaleHeight * $lightRange] } else { set light1y $y }
   # adjust light normalization to scale
   set light1yn [expr $light1y / $scaleHeight]
   drawLights
}

proc setLight2x {x} {
   global light2x scaleWidth

   # limit light position to width of terrain
   if { $x > $scaleWidth } { set light2x $scaleWidth } else { set light2x $x }
   drawLights
}
proc setLight2z {z} {
   global light2z scaleLength

   # limit light position to length of terrain
   if { $z > $scaleLength } { set light2z $scaleLength } else { set light2z $z }
   drawLights
}
proc setLight2y {y} {
   global light2y light2yn lightRange scaleHeight

   # limit-to-max rule
   if { $y > [expr $scaleHeight * $lightRange] } { set light2y [expr $scaleHeight * $lightRange] } else { set light2y $y }
   # adjust light normalization to scale
   set light2yn [expr $light2y / $scaleHeight]
   drawLights
}

proc setLight3x {x} {
   global light3x scaleWidth

   # limit light position to width of terrain
   if { $x > $scaleWidth } { set light3x $scaleWidth } else { set light3x $x }
   drawLights
}
proc setLight3z {z} {
   global light3z scaleLength

   # limit light position to length of terrain
   if { $z > $scaleLength } { set light3z $scaleLength } else { set light3z $z }
   drawLights
}
proc setLight3y {y} {
   global light3y light3yn lightRange scaleHeight

   # limit-to-max rule
   if { $y > [expr $scaleHeight * $lightRange] } { set light3y [expr $scaleHeight * $lightRange] } else { set light3y $y }
   # adjust light normalization to scale
   set light3yn [expr $light3y / $scaleHeight]
   drawLights
}

# Light controls
#
frame    .fCtrlLight          -background $rgbLightLt  -borderwidth $spFrame   \
                              -width $xFrame -height $yFrame -relief $relFrame

label    .fCtrlLight.lLtPos   -background $rgbLightLt  -text "Light positions"
label    .fCtrlLight.lx1      -background $rgbLightLt  -text "Light 1 x "
entry    .fCtrlLight.ex1      -background $colorText  -textvariable light1x -width $widthText
label    .fCtrlLight.lz1      -background $rgbLightLt  -text "Light 1 z "
entry    .fCtrlLight.ez1      -background $colorText  -textvariable light1z -width $widthText

label    .fCtrlLight.lx2      -background $rgbLightLt  -text "Light 2 x "
entry    .fCtrlLight.ex2      -background $colorText  -textvariable light2x -width $widthText
label    .fCtrlLight.lz2      -background $rgbLightLt  -text "Light 2 z "
entry    .fCtrlLight.ez2      -background $colorText  -textvariable light2z -width $widthText

label    .fCtrlLight.lx3      -background $rgbLightLt  -text "Light 3 x "
entry    .fCtrlLight.ex3      -background $colorText  -textvariable light3x -width $widthText
label    .fCtrlLight.lz3      -background $rgbLightLt  -text "Light 3 z "
entry    .fCtrlLight.ez3      -background $colorText  -textvariable light3z -width $widthText

scale    .fCtrlLight.sLight1x   -background $rgbLightLt \
                                 -command "setLight1x " -label "x left-right"  \
                                 -showvalue 1 -from   0 -to [expr $scaleWidth * 2] -resolution 0.10 \
                                 -orient horizontal -troughcolor $rgbLightDk \
                                 -variable light1x -width $widthScale

frame    .fCtrlLight.fLight1ZY   -background $rgbLightLt
scale    .fCtrlLight.fLight1ZY.sLight1z   -background $rgbLightLt \
                                 -command "setLight1z " -label "z"  \
                                 -showvalue 1 -from [expr $scaleLength * 2] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbLightDk \
                                 -variable light1z -length $lengthZY -width $widthScale

scale    .fCtrlLight.fLight1ZY.sLight1y   -background $rgbLightLt \
                                 -command "setLight1y " -label "y"  \
                                 -showvalue 1 -from [expr $scaleHeight * $lightRange] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbLightDk \
                                 -variable light1y -length $lengthZY -width $widthScale

label    .fCtrlLight.fLight1ZY.lZY -background $rgbLightLt \
            -text "1 - Main\n     light:\n  y\nlow-high\n  z\nnear-far" -justify left

pack configure \
   .fCtrlLight.fLight1ZY.lZY         \
   .fCtrlLight.fLight1ZY.sLight1y   \
   .fCtrlLight.fLight1ZY.sLight1z   \
   -side left -fill x -anchor "w" -expand 0

scale    .fCtrlLight.sLight2x   -background $rgbLightLt \
                                 -command "setLight2x " -label "x left-right"  \
                                 -showvalue 1 -from   0 -to [expr $scaleWidth * 2] -resolution 0.10 \
                                 -orient horizontal -troughcolor $rgbLightDk \
                                 -variable light2x -width $widthScale

frame    .fCtrlLight.fLight2ZY   -background $rgbLightLt
scale    .fCtrlLight.fLight2ZY.sLight2z   -background $rgbLightLt \
                                 -command "setLight2z " -label "z"  \
                                 -showvalue 1 -from [expr $scaleLength * 2] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbLightDk \
                                 -variable light2z -length $lengthZY -width $widthScale

scale    .fCtrlLight.fLight2ZY.sLight2y   -background $rgbLightLt \
                                 -command "setLight2y " -label "y"  \
                                 -showvalue 1 -from [expr $scaleHeight * $lightRange] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbLightDk \
                                 -variable light2y -length $lengthZY -width $widthScale

label    .fCtrlLight.fLight2ZY.lZY -background $rgbLightLt \
            -text "2 - On\ncamera:\n  y\nlow-high\n  z\nnear-far" -justify left

pack configure \
   .fCtrlLight.fLight2ZY.lZY        \
   .fCtrlLight.fLight2ZY.sLight2y   \
   .fCtrlLight.fLight2ZY.sLight2z   \
   -side left -fill x -anchor "w" -expand 0

scale    .fCtrlLight.sLight3x   -background $rgbLightLt \
                                 -command "setLight3x " -label "x left-right"  \
                                 -showvalue 1 -from   0 -to [expr $scaleWidth * 2] -resolution 0.10 \
                                 -orient horizontal -troughcolor $rgbLightDk \
                                 -variable light3x -width $widthScale

frame    .fCtrlLight.fLight3ZY   -background $rgbLightLt
scale    .fCtrlLight.fLight3ZY.sLight3z   -background $rgbLightLt    \
                                 -command "setLight3z " -label "z"   \
                                 -showvalue 1 -from [expr $scaleLength * 2] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbLightDk \
                                 -variable light3z -length $lengthZY -width $widthScale

scale    .fCtrlLight.fLight3ZY.sLight3y   -background $rgbLightLt    \
                                 -command "setLight3y " -label "y"   \
                                 -showvalue 1 -from [expr $scaleHeight * $lightRange] -to 0 -resolution 0.10 \
                                 -orient vertical -troughcolor $rgbLightDk    \
                                 -variable light3y -length $lengthZY -width $widthScale

label    .fCtrlLight.fLight3ZY.lZY -background $rgbLightLt \
            -text "3 - Back\n     light:\n  y\nlow-high\n  z\nnear-far" -justify left

pack configure \
   .fCtrlLight.fLight3ZY.lZY        \
   .fCtrlLight.fLight3ZY.sLight3y   \
   .fCtrlLight.fLight3ZY.sLight3z   \
   -side left -fill x -anchor "w" -expand 0

checkbutton .fCtrlLight.cLightLock  -background $rgbLightLt \
                           -text "lock" -variable lightLock \
                           -anchor "w"
                           # -height 1

button   .fCtrlLight.bUndo   -background $rgbLightLt  -command "ctrlLightUndo" -text "Undo "  \
                              -width $widthBut
button   .fCtrlLight.bDone   -background $rgbLight    -command "ctrlLightDone"  -foreground White -text "Done "  \
                              -width $widthBut
button   .fCtrlLight.bExit   -background $rgbLightLt  -command exit  -text "Exit "  \
                              -width $widthBut

label    .fCtrlLight.lBlank0  -background $rgbLightLt  -text " "
label    .fCtrlLight.lBlank1  -background $rgbLightLt  -text " "
label    .fCtrlLight.lBlank2  -background $rgbLightLt  -text " "
label    .fCtrlLight.lBlank3  -background $rgbLightLt  -text " "
label    .fCtrlLight.lBlank4  -background $rgbLightLt  -text " "

label    .fCtrlLight.lSpace0  -text " "
label    .fCtrlLight.lSpace1  -text " "
label    .fCtrlLight.lSpace2  -text " "
label    .fCtrlLight.lSpace3  -text " "

#    .fCtrlLight.lLtPos
pack configure \
   .fCtrlLight.fLight1ZY   .fCtrlLight.sLight1x \
   .fCtrlLight.fLight2ZY   .fCtrlLight.sLight2x \
   .fCtrlLight.fLight3ZY   .fCtrlLight.sLight3x \
   .fCtrlLight.bUndo    \
   .fCtrlLight.bDone    \
   .fCtrlLight.lBlank0  \
   -side top -fill x -anchor "nw" -expand 0

# -------------------------
# --- File applications ---

# File application variables
#
variable fileTgaHf1 fileTgaHf2 fileTgaHfOut
variable fileGifIO fileTgaIO

# File defaults
#
set fileTgaHf1    "hfa.tga"
set fileTgaHf2    "hfb.tga"
set fileTgaHfOut  "hfout.tga"
set fileGifIO     "gifio.gif"
set fileTgaIO     "tgaio.tga"
set valFileM 1
set valFileV 15

proc setFileV {v} {
   global valFileV
   set valFileV $v
}
proc cmdGif2Tga {} {
   global fileGifIO fileTgaIO
   exec gif2tga $fileGifIO $fileTgaIO &
   puts "\nGif2Tga spawned by GT"
}
proc cmdTga2Gif {} {
   global fileTgaIO fileGifIO
   exec tga2gif -i$fileTgaIO -o$fileGifIO &
   puts "\nTga2Gif spawned by GT"
}
proc cmdHfMerge {} {
   global fileTgaHf1 fileTgaHf2 fileTgaHfOut
   global valFileV valFileM

   if {valFileM > 5} {
      exec hfmerge -a$fileTgaHf1 -b$fileTgaHf2 -o$fileTgaHfOut -v$valFileV -m$valFileM &
   puts "\nHFmerge FileMorph(a) spawned by GT"
   } else {
      exec hfmerge -a$fileTgaHf1 -b$fileTgaHf2 -o$fileTgaHfOut -m$valFileM &
   puts "\nHFmerge FileOp(a,b) spawned by GT"
   }
}

# File operations controls
#
frame    .fCtrlFile -background $rgbFileOp -width $xFrame -height $yFrame -borderwidth $spFrame -relief $relFrame
label    .fCtrlFile.lFileHfMerge    -background $rgbFileOp  -text "HeightField Merge TGAs"
label    .fCtrlFile.lFileTgaHf1     -background $rgbFileOp  -text "a TGA heightfield in: "
entry    .fCtrlFile.eFileTgaHf1     -background $colorText  -textvariable fileTgaHf1   -width $widthText
label    .fCtrlFile.lFileTgaHf2     -background $rgbFileOp  -text "b TGA heightfield in: "
entry    .fCtrlFile.eFileTgaHf2     -background $colorText  -textvariable fileTgaHf2   -width $widthText
label    .fCtrlFile.lFileTgaHfOut   -background $rgbFileOp  -text "o TGA heightfield out: "
entry    .fCtrlFile.eFileTgaHfOut   -background $colorText  -textvariable fileTgaHfOut -width $widthText

scale    .fCtrlFile.sFileV          -background $rgbFileOp -foreground Black           \
                                    -troughcolor $rgbFileOpDk -label "v % morph of file a"   \
                                    -orient horizontal -width $widthScale -command "setFileV"            \
                                    -showvalue 1   \
                                    -from 1 -to 100 -variable valFileV

button   .fCtrlFile.bHFmerge        -background $rgbFileOpB -command exit  -text "HFmerge"   -width $widthBut

label    .fCtrlFile.lFileXlate      -background $rgbFileOp  -text "GIFs <- Translate -> TGAs"
label    .fCtrlFile.lFileGif        -background $rgbFileOp  -text "GIF I/O file: "
entry    .fCtrlFile.eFileGif        -background $colorText  -textvariable fileGifIO -width $widthText
label    .fCtrlFile.lFileTga        -background $rgbFileOp  -text "TGA I/O file: "
entry    .fCtrlFile.eFileTga        -background $colorText  -textvariable fileTgaIO -width $widthText

button   .fCtrlFile.bGif2Tga  -background $rgbFileOpB -command exit  -text "Gif2Tga"   -width $widthBut
button   .fCtrlFile.bTga2Gif  -background $rgbFileOpB -command exit  -text "Tga2Gif"   -width $widthBut
button   .fCtrlFile.bExit     -background $rgbFileOp  -command exit  -text "Exit "      -width $widthBut

# duplicates are needed because only one-instance-per is legal
#
label    .fCtrlFile.lFileBlank0     -background $rgbFileOp -text " "
label    .fCtrlFile.lFileBlank1     -background $rgbFileOp -text " "
label    .fCtrlFile.lFileBlank2     -background $rgbFileOp -text " "
label    .fCtrlFile.lFileBlank3     -background $rgbFileOp -text " "
label    .fCtrlFile.lFileBlank4     -background $rgbFileOp -text " "
label    .fCtrlFile.lFileBlank5     -background $rgbFileOp -text " "

label    .fCtrlFile.lFileSpace0     -text " "
label    .fCtrlFile.lFileSpace1     -text " "

frame .fCtrlFile.fRadio -background $rgbFileOp

frame .fCtrlFile.fRadio.fRadioL -background $rgbFileOp
radiobutton .fCtrlFile.fRadio.fRadioL.rFileM01  -background $rgbFileOp -text "avg (a, b)"   -variable valFileM -value  1
radiobutton .fCtrlFile.fRadio.fRadioL.rFileM02  -background $rgbFileOp -text "  a + b   "   -variable valFileM -value  2
radiobutton .fCtrlFile.fRadio.fRadioL.rFileM03  -background $rgbFileOp -text "  a - b   "   -variable valFileM -value  3
radiobutton .fCtrlFile.fRadio.fRadioL.rFileM04  -background $rgbFileOp -text "max (a, b)"   -variable valFileM -value  4
radiobutton .fCtrlFile.fRadio.fRadioL.rFileM05  -background $rgbFileOp -text "min (a, b)"   -variable valFileM -value  5
pack configure \
   .fCtrlFile.fRadio.fRadioL.rFileM01  \
   .fCtrlFile.fRadio.fRadioL.rFileM02  \
   .fCtrlFile.fRadio.fRadioL.rFileM03  \
   .fCtrlFile.fRadio.fRadioL.rFileM04  \
   .fCtrlFile.fRadio.fRadioL.rFileM05  \
   -side top -fill y -anchor "w" -expand 1

frame .fCtrlFile.fRadio.fRadioR -background $rgbFileOp
radiobutton .fCtrlFile.fRadio.fRadioR.rFileM06  -background $rgbFileOp -text "a + v%   up"   -variable valFileM -value  6
radiobutton .fCtrlFile.fRadio.fRadioR.rFileM07  -background $rgbFileOp -text "a - v% down"   -variable valFileM -value  7
radiobutton .fCtrlFile.fRadio.fRadioR.rFileM08  -background $rgbFileOp -text "a below v%"   -variable valFileM -value  8
radiobutton .fCtrlFile.fRadio.fRadioR.rFileM09  -background $rgbFileOp -text "a above v%"   -variable valFileM -value  9
radiobutton .fCtrlFile.fRadio.fRadioR.rFileM10  -background $rgbFileOp -text "a scale v%"   -variable valFileM -value 10
radiobutton .fCtrlFile.fRadio.fRadioR.rFileM11  -background $rgbFileOp -text "a+rand(v%)"   -variable valFileM -value 11
pack configure \
   .fCtrlFile.fRadio.fRadioR.rFileM06  \
   .fCtrlFile.fRadio.fRadioR.rFileM07  \
   .fCtrlFile.fRadio.fRadioR.rFileM08  \
   .fCtrlFile.fRadio.fRadioR.rFileM09  \
   .fCtrlFile.fRadio.fRadioR.rFileM10  \
   .fCtrlFile.fRadio.fRadioR.rFileM11  \
   -side top -fill y -anchor "w" -expand 1

pack configure \
   .fCtrlFile.fRadio.fRadioL  \
   .fCtrlFile.fRadio.fRadioR  \
   -side left -fill y -anchor "nw" -expand 1

pack configure \
   .fCtrlFile.lFileTgaHf1     .fCtrlFile.eFileTgaHf1     \
   .fCtrlFile.lFileTgaHf2     .fCtrlFile.eFileTgaHf2     \
   .fCtrlFile.lFileTgaHfOut   .fCtrlFile.eFileTgaHfOut   \
   .fCtrlFile.lFileBlank0  \
   .fCtrlFile.fRadio       \
   .fCtrlFile.sFileV       \
   .fCtrlFile.bHFmerge     \
   .fCtrlFile.lFileBlank1  \
   .fCtrlFile.lFileGif        .fCtrlFile.eFileGif        \
   .fCtrlFile.lFileTga        .fCtrlFile.eFileTga        \
   .fCtrlFile.lFileBlank2  \
   .fCtrlFile.bGif2Tga     \
   .fCtrlFile.bTga2Gif     \
   .fCtrlFile.lFileBlank3  \
   -side top -fill x -anchor "nw" -expand 0

# --- CUTTER application

# CUTTER    cuts away all objects that are invisible to the POVRAY-camera.
# -i  name of the INC-file, including all objects
# -o  name of the output-INC-file, the visible objects
# -p  name of the POV-file, including the camera declaration
# -e  max distance limit (0 no limit, 1...~ value for limit)
# -s  min distance limit (0 no limit, 1...~ value for limit)

set maxCut [expr sqrt(($scaleWidth * $scaleWidth) + ($scaleLength * $scaleLength))]
set valCutE [expr $maxCut * 0.75]
set valCutS [expr $maxCut * 0.25]

set valCutObji    2
set valCutObjf    0.75
set valCutObj [expr $valCutObji + $valCutObjf]

set boolSymbols 1
set fileCutterIncludeOut   "gtcut.inc"
set fileCutterScaleOut     "gtcut_s.inc"

proc setCutE {e} {
   global valE
   set valCutE $e
   # puts stdout $valCutE
}
proc setCutS {s} {
   global valCutS
   set valCutS $s
   # puts stdout $valCutS
}
proc setCutObj {} {
   global valCutObj valCutObji valCutObjf
   set valCutObj [expr $valCutObji + $valCutObjf]
}
proc setCutObji {i} {
   global valCutObji
   set valCutObji $i
   setCutObj
}
proc setCutObjf {f} {
   global valCutObjf
   set valCutObjf $f
   setCutObj
}

# Cutter itself reads files, writes to stdout and files.
# Simple execution does not work because of console blocking.
# Redirection never worked properly.
# Spawning appears to work, but subsequent filedata-consumers starve.
#
# Catching execution works great, output "$data" is just discarded.
#
proc cmdCutter {} {
   global valCutE valCutS
   global fileCameraPov fileGenIncludeOut fileCutterIncludeOut fileCutterScaleOut
   global xlocGIF ylocGIF imageRough

   catch { exec cutter \
      -i$fileGenIncludeOut \
      -o$fileCutterIncludeOut \
      -p$fileCameraPov \
      -e$valCutE \
      -s$valCutS } data

   puts "\nCutter executed by GT"

   # wipe, then immediately redisplay topo map so user doesn't see any blink
   #
   .fMisc.cGraphic delete all
   .fMisc.cGraphic create image $xlocGIF $ylocGIF -image $imageRough

   # Following overlays redraw over the GIF,
   # trees are parsed out of Gen-created POV-Ray include file.
   # (dataReadGen calls draw for incremental display)
   #
   dataReadGen $fileCutterIncludeOut $fileCutterScaleOut
   puts stdout "\nCutter dataReadGen done, executed by GT"

   # camera, Red line with arrow
   #
   drawCamera
   puts stdout "\nCutter drawCamera done, executed by GT"

   # lights, Blue circles
   #
   drawLights
   puts stdout "\nCutter drawLights done, executed by GT"
}

# --- control widgets for CUTTER app ---

frame .fCtrlCutter -background $rgbCutter -width $xFrame -height $yFrame -borderwidth $spFrame -relief $relFrame
label    .fCtrlCutter.lCutIncIn     -background $rgbCutter -foreground Black -text "i Pov include in: "
entry    .fCtrlCutter.eCutIncIn     -background $colorText -width $widthText -textvariable fileGenIncludeOut
label    .fCtrlCutter.lCutIncOut    -background $rgbCutter -foreground Black -text "o Pov include out: "
entry    .fCtrlCutter.eCutIncOut    -background $colorText -width $widthText -textvariable fileCutterIncludeOut
label    .fCtrlCutter.lCutPovCam    -background $rgbCutter -foreground Black -text "p Pov camera file: "
entry    .fCtrlCutter.eCutPovCam    -background $colorText -width $widthText -textvariable fileCameraPov

radiobutton .fCtrlCutter.rUseSymbols   -background $rgbCutter -text "use symbols"  -anchor "w"  \
                                       -variable boolSymbols -value 1
radiobutton .fCtrlCutter.rUseLiterals  -background $rgbCutter -text "use literals" -anchor "w"  \
                                       -variable boolSymbols -value 0

scale    .fCtrlCutter.sCutE         -background $rgbCutter -foreground Black                 \
                                    -troughcolor $rgbCutterDk -label "e max distance"        \
                                    -orient horizontal -width $widthScale -command "setCutE" \
                                    -showvalue 1 -from 0 -to $maxCut -variable valCutE

scale    .fCtrlCutter.sCutS         -background $rgbCutter -foreground Black                 \
                                    -troughcolor $rgbCutterDk -label "s min distance"        \
                                    -orient horizontal -width $widthScale -command "setCutS" \
                                    -showvalue 1 -from  0 -to $maxCut -variable valCutS

label    .fCtrlCutter.lCutObjInc    -background $rgbCutter -foreground Black -text "scaled Pov include out: "
entry    .fCtrlCutter.eCutObjInc    -background $colorText -width $widthText -textvariable fileCutterScaleOut
scale    .fCtrlCutter.sCutObji      -background $rgbCutter -foreground Black                 \
                                    -troughcolor $rgbCutterDk -label "integer, Obj scale"    \
                                    -orient horizontal -width $widthScale -command "setCutObji" \
                                    -showvalue 1 -from  0 -to 10 -variable valCutObji

scale    .fCtrlCutter.sCutObjf      -background $rgbCutter -foreground Black                 \
                                    -troughcolor $rgbCutterDk -label "fraction, Obj scale"   \
                                    -orient horizontal -width $widthScale -command "setCutObjf" \
                                    -resolution 0.05 \
                                    -showvalue 1 -from  0.00 -to 0.95 -variable valCutObjf


button   .fCtrlCutter.bCutter       -background $rgbCutterB -foreground Black -text "Cutter" -command "cmdCutter"

label    .fCtrlCutter.lCutBlank0    -background $rgbCutter -text " "
label    .fCtrlCutter.lCutBlank1    -background $rgbCutter -text " "
label    .fCtrlCutter.lCutBlank2    -background $rgbCutter -text " "
label    .fCtrlCutter.lCutBlank3    -background $rgbCutter -text " "

# frame .fCtrlCutter.fSwitchSymbols -background $rgbCutter -width $xFrame -height $yFrame -borderwidth $spFrame -relief $relFrame
# radiobutton .fCtrlCutter.fSwitchSymbols.rUseSymbols  -background $rgbCutter -text "use symbols" -variable boolSymbols -value 1
# radiobutton .fCtrlCutter.fSwitchSymbols.rUseLiterals -background $rgbCutter -text "use literals" -variable boolSymbols -value 0
# pack configure \
#    .fCtrlCutter.fSwitchSymbols.rUseSymbols   \
#    .fCtrlCutter.fSwitchSymbols.rUseLiterals  \
#    -side left -fill x -anchor "nw" -expand 1

pack configure \
   .fCtrlCutter.lCutIncIn  \
   .fCtrlCutter.eCutIncIn  \
   .fCtrlCutter.lCutIncOut \
   .fCtrlCutter.eCutIncOut \
   .fCtrlCutter.lCutPovCam \
   .fCtrlCutter.eCutPovCam \
   .fCtrlCutter.lCutBlank0 \
   .fCtrlCutter.sCutE      \
   .fCtrlCutter.sCutS      \
   .fCtrlCutter.lCutBlank1 \
   .fCtrlCutter.lCutObjInc \
   .fCtrlCutter.eCutObjInc \
   .fCtrlCutter.lCutBlank2 \
   .fCtrlCutter.sCutObji   \
   .fCtrlCutter.sCutObjf   \
   .fCtrlCutter.bCutter    \
   .fCtrlCutter.lCutBlank3 \
   -side top -fill x -anchor "nw" -expand 1

# --- GEN application ---
# GEN       places objects onto the surface of a heightfield.
# -i input TGA heightfield filename
# -o output INClude filename - file will have locations of "myBigTree"s
# -m plantmap filename, the GIF made by ROUGH
# -n object name, a placeholder like "myBigTree"
# -p palette entry for plants
# -s space between objects
# -x -y -z dimension ( coordinate at corner opposite <0, 0, 0> )
# -d lower altitude bound for objects
# -u upper altitude bound for objects
# -rp random placement switch
# -rr random rotation switch
# -rs random scaling
# -rd random seed, default 1

# GEN application variables
#
variable fileGenTgaHfIn
variable fileGenPlantMapIn
variable nameGenObject
variable fileGenIncludeOut
variable fileGenScaleOut

variable dataGen
variable valGenP valGenS valGenD valGenU
variable valGenX valGenY valGenZ
variable valGenRP valGenRR valGenRS valGenRD
variable xVal
variable yVal
variable zVal
variable xfVal
variable dListGen
variable dIndex

# GEN defaults
#
set fileGenTgaHfIn      "gfoutput.tga"
set fileGenPlantMapIn   "gttest.gif"
set nameGenObject       "Fern"
set fileGenIncludeOut   "gttest.inc"
set fileGenScaleOut     "gttest_s.inc"

set valGenS     2
set valGenD     2
set valGenU    95
set valGenRP    1
set valGenRR    1
set valGenRS   80
set valGenRD    3

# topo map level-select radio button
#
# e ->  = 1
# d..e  = 6
# c..d  = 5
# b..c  = 4
# a..b  = 3
# <- a  = 2
#
set valGenP 5

# already set in camera.inc
# scaleWidth
# scaleLength
# scaleHeight

set scaleWidthOld    $scaleWidth
set scaleLengthOld   $scaleLength
set scaleHeightOld   $scaleHeight

# GEN application methods
#
proc setGenS {s} {
   global valGenS
   set valGenS $s
   # puts stdout $valGenS
}
proc setGenD {d} {
   global valGenD
   set valGenD $d
   # puts stdout $valGenD
}
proc setGenU {u} {
   global valGenU
   set valGenU $u
   # puts stdout $valGenU
}
proc setGenRS {rs} {
   global valGenRS
   set valGenRS $rs
   # puts stdout $valGenRS
}
proc setGenRD {rd} {
   global valGenRD
   set valGenRD $rd
   # puts stdout $valGenRD
}

# Only allow set Gen X, Z, and Y while in modalActive state,
# meaning Camera or Lights settings are pending.  Those are
# the only guaranteed parameter file synchronization states.
#
proc setGenX {x} {
   global modalActive scaleWidth scaleWidthOld
   global cameraX cameraXn cameraLookX cameraLookXn
   global light1x light1xn light2x light2xn light3x light3xn

   if { $modalActive == "active" } {
      set scaleWidth $x
      set cameraX       [ expr $scaleWidth * $cameraXn ]
      set cameraLookX   [ expr $scaleWidth * $cameraLookXn ]
      set light1x       [ expr $scaleWidth * $light1xn ]
      set light2x       [ expr $scaleWidth * $light2xn ]
      set light3x       [ expr $scaleWidth * $light3xn ]
      set scaleWidthOld $scaleWidth
   } else { set scaleWidth $scaleWidthOld }
}
proc setGenZ {z} {
   global modalActive scaleLength scaleLengthOld
   global cameraZ cameraZn cameraLookZ cameraLookZn
   global light1z light1zn light2z light2zn light3z light3zn

   if { $modalActive == "active" } {
      set scaleLength $z
      set cameraZ       [ expr $scaleLength * $cameraZn ]
      set cameraLookZ   [ expr $scaleLength * $cameraLookZn ]
      set light1z       [ expr $scaleLength * $light1zn ]
      set light2z       [ expr $scaleLength * $light2zn ]
      set light3z       [ expr $scaleLength * $light3zn ]
      set scaleLengthOld $scaleLength
   } else { set scaleLength $scaleLengthOld }
}
proc setGenY {y} {
   global modalActive scaleHeight scaleHeightOld
   global cameraY cameraYn cameraLookY cameraLookYn
   global light1y light1yn light2y light2yn light3y light3yn

   if { $modalActive == "active" } {
      set scaleHeight $y
      set cameraY       [ expr $scaleHeight * $cameraYn ]
      set cameraLookY   [ expr $scaleHeight * $cameraLookYn ]
      set light1y       [ expr $scaleHeight * $light1yn ]
      set light2y       [ expr $scaleHeight * $light2yn ]
      set light3y       [ expr $scaleHeight * $light3yn ]
      set scaleHeightOld $scaleHeight
   } else { set scaleHeight $scaleHeightOld }
}

# dataDrawGen
# Parse and translate data into canvas draw statements (as-slow-as 1 object).
# Receives one line of POV-Ray script from gttest.inc - passed by dataReadGen.
#
proc dataDrawGen {dGen} {
   global xVal
   global xfVal
   global yVal
   global zVal
   global dListGen
   global dIndex
   global widthGIFf heightGIFf
   global scaleWidth scaleLength treeRadius

   global valCutObj
   global dataGen

   variable searchTarget
   variable dCopyGen

   # ACHTUNG: initialize variables to junk float values (in this case),
   # OR ELSE expr will not understand them - even after subsequent
   # assignments that appear okay EVEN using puts (because you see a string)!
   # Failure to initialize caused total code failures...
   # .. with mystery error reports - an added BONUS.
   #
   set xVal 10.123456
   set yVal 10.123456
   set zVal 10.123456

   # working parser
   # decimate all special characters to single spaces
   # TBD - make this into one expression using a list of chars
   # regsub -all --  "{"  $dGen      " " dCopyGen
   # regsub -all --  "<"  $dCopyGen  " " dGen
   # regsub -all --  ","  $dGen      " " dCopyGen
   # regsub -all --  ">"  $dCopyGen  " " dGen
   # regsub -all --  "}"  $dGen      " " dCopyGen
   # regsub -all -- "  "  $dCopyGen  " " dGen
   # puts "$dGen "

   ### scale handler
   # find scale string in list
   set s [string first "scale" $dGen]

   if { $s != -1 } {
      # point past first scale string, 6 is just past "<"
      set s [expr $s + 6]

      # fetch substring (three scale floats are at the beginning)
      set xyz [string range $dGen $s end]

      # assign xyz size (scale)
      scan $xyz "%f,%f,%f" xVal yVal zVal
      set xVal [ expr $xVal * $valCutObj ]
      set yVal [ expr $yVal * $valCutObj ]
      set zVal [ expr $zVal * $valCutObj ]

      ### translate handler
      # find translate string in list
      set t [string first "translate" $dGen]

      # construct complete newly scaled line
      set s [expr $s - 1]
      set t [expr $t - 2 ]
      set dCopyGen "[string range $dGen 0 $s]$xVal,$yVal,$zVal[string range $dGen $t end]"
      # puts $dCopyGen
      set t [expr $t + 2]

      # point past first translate string, 10 is just past "<"
      # set t [expr $t + 9]
      set t [expr $t + 10]

      # fetch substring (three translate floats are at the beginning)
      set xyz [string range $dGen $t end]

      # assign xyz coordinates (translate)
      # old, works with precleaned string
      # assign floating-point xyz coordinates
      # scan $xyz "%f %f %f" xVal yVal zVal
      #
      # new, works with uncleaned string
      scan $xyz "%f,%f,%f" xVal yVal zVal

      set xLo [expr                 (( $widthGIFf  / $scaleWidth  ) * $xVal )  - ( $treeRadius )]
      set zLo [expr ( $heightGIFf - (( $heightGIFf / $scaleLength ) * $zVal )) + ( $treeRadius )]
      set xHi [expr                 (( $widthGIFf  / $scaleWidth  ) * $xVal )  + ( $treeRadius )]
      set zHi [expr ( $heightGIFf - (( $heightGIFf / $scaleLength ) * $zVal )) - ( $treeRadius )]
      # puts "$xLo $zLo $xHi $zHi "

      # draw with display
      .fMisc.cGraphic create oval $xLo $zLo $xHi $zHi -fill Green

      # copy newly scaled data
      set dataGen $dCopyGen
   }
}
# dataReadGen
# Read in object locations from POV-Ray script gttest.inc, made by Gen,
# one line at a time, then call drawGen for each (object) line item.
# OR
# Read in object locations from POV-Ray script gtcut.inc, made by Cutter,
# one line at a time, then call drawGen for each (object) line item.
#
proc dataReadGen {fn fo} {
   global dataGen
#   global fileGenIncludeOut

   # open file for read
   # set f [ open $fileGenIncludeOut ]
   set fin  [ open $fn ]
   set fout [ open $fo w ]
   # go parse and display objects
   # as-slow-as n objects
   #
   while { [ gets $fin dataGen ] != -1 } {
      dataDrawGen $dataGen
      puts $fout $dataGen
      # attempt realtime display after creation of each object...
      # ... does not work, still no display 'til all objects are done!
      # make sure gets is working line-at-a-time?
      pack .fMisc.cGraphic
   }

   # close file after read
   close $fin

   # close file to complete write
   close $fout
}
proc cmdGen {} {
   global imageRough
   global dataGen
   global fileGenTgaHfIn fileGenIncludeOut fileGenScaleOut
   global fileGenPlantMapIn nameGenObject
   global fileRoughGifMapOut
   global valGenP
   global valGenS valGenD valGenU
   global valGenRP valGenRR valGenRS valGenRD
   global xlocGIF ylocGIF
   global scaleWidth scaleLength scaleHeight

   # Don't spawn processes in the output-processing stream!
   # Consumers may starve or eat old data.
   # Gen produces a data file for susbsequent display.

   # puts stdout "\ngen -i$fileGenTgaHfIn    \

   exec gen -i$fileGenTgaHfIn    \
                    -o$fileGenIncludeOut \
                    -m$fileGenPlantMapIn \
                    -n$nameGenObject     \
                    -p$valGenP           \
                    -s$valGenS           \
                    -x$scaleWidth        \
                    -y$scaleHeight       \
                    -z$scaleLength       \
                    -d$valGenD           \
                    -u$valGenU           \
                    -rp$valGenRP         \
                    -rr$valGenRR         \
                    -rs$valGenRS         \
                    -rd$valGenRD"

   puts stdout "\nGen done, executed by GT"

   # wipe, then immediately redisplay topo map so user doesn't see any blink
   #
   .fMisc.cGraphic delete all
   .fMisc.cGraphic create image $xlocGIF $ylocGIF -image $imageRough

   # Following overlays redraw over the GIF,
   # trees are parsed out of Gen-created POV-Ray include file.
   # (dataReadGen calls draw for incremental display)
   #
   dataReadGen $fileGenIncludeOut $fileGenScaleOut
   puts stdout "\nGen dataReadGen done, executed by GT"

   # camera, Red line with arrow
   #
   drawCamera
   puts stdout "\nGen drawCamera done, executed by GT"

   # lights, Blue circles
   #
   drawLights
   puts stdout "\nGen drawLights done, executed by GT"
}

# GEN application controls
#
frame    .fCtrlGen               -background $rgbGen     -borderwidth $spFrame  -relief $relFrame
label    .fCtrlGen.lGenTgaHfIn   -background $rgbGen     -text "i Tga heightfield in: "
entry    .fCtrlGen.eGenTgaHfIn   -background $colorText  -textvariable fileGenTgaHfIn     -width $widthText
label    .fCtrlGen.lGenPlantMap  -background $rgbGen     -text "m Gif plant map in: "
entry    .fCtrlGen.eGenPlantMap  -background $colorText  -textvariable fileGenPlantMapIn  -width $widthText
label    .fCtrlGen.lGenObjName   -background $rgbGen     -text "n Object name: "
entry    .fCtrlGen.eGenObjName   -background $colorText  -textvariable nameGenObject      -width $widthText
label    .fCtrlGen.lGenIncOut    -background $rgbGen     -text "o Pov include out: "
entry    .fCtrlGen.eGenIncOut    -background $colorText  -textvariable fileGenIncludeOut  -width $widthText

scale .fCtrlGen.sGenU   -background $rgbGen  -command "setGenU" -label "u up to"    \
                        -showvalue 1 -from   0 -to 100   -orient horizontal \
                        -troughcolor $rgbGenDk -variable valGenU -width $widthScale
scale .fCtrlGen.sGenS   -background $rgbGen  -command "setGenS" -label "s spacer"   \
                        -showvalue 1 -from   1 -to 10    -orient horizontal \
                        -troughcolor $rgbGenDk -variable valGenS -width $widthScale

frame .fCtrlGen.fRadioSlide      -background $rgbGen

frame .fCtrlGen.fRadioSlide.fRadioLevel   -background $rgbGen
label       .fCtrlGen.fRadioSlide.fRadioLevel.lGenHt     -background $rgbGen  -text " height  "
radiobutton .fCtrlGen.fRadioSlide.fRadioLevel.rGenP1     -background $rgbGen  -text " e ->  "  -variable valGenP -value 1
radiobutton .fCtrlGen.fRadioSlide.fRadioLevel.rGenP6     -background $rgbGen  -text " d..e  "  -variable valGenP -value 6
radiobutton .fCtrlGen.fRadioSlide.fRadioLevel.rGenP5     -background $rgbGen  -text " c..d  "  -variable valGenP -value 5
radiobutton .fCtrlGen.fRadioSlide.fRadioLevel.rGenP4     -background $rgbGen  -text " b..c  "  -variable valGenP -value 4
radiobutton .fCtrlGen.fRadioSlide.fRadioLevel.rGenP3     -background $rgbGen  -text " a..b  "  -variable valGenP -value 3
radiobutton .fCtrlGen.fRadioSlide.fRadioLevel.rGenP2     -background $rgbGen  -text " <- a  "  -variable valGenP -value 2

pack configure \
   .fCtrlGen.fRadioSlide.fRadioLevel.lGenHt  \
   .fCtrlGen.fRadioSlide.fRadioLevel.rGenP1  \
   .fCtrlGen.fRadioSlide.fRadioLevel.rGenP6  \
   .fCtrlGen.fRadioSlide.fRadioLevel.rGenP5  \
   .fCtrlGen.fRadioSlide.fRadioLevel.rGenP4  \
   .fCtrlGen.fRadioSlide.fRadioLevel.rGenP3  \
   .fCtrlGen.fRadioSlide.fRadioLevel.rGenP2  \
   -side top -fill y -anchor "s"
   # better presentation vertically centered, without anchor
   # -anchor "n"
   # -expand 1


frame       .fCtrlGen.fRadioSlide.fUpRandDn  -background $rgbGen
scale       .fCtrlGen.fRadioSlide.fUpRandDn.sGenRD -background $rgbGen -command "setGenRD" -label "rd random seed"   \
                                       -showvalue 1 -from   1 -to  10   -orient horizontal \
                                       -troughcolor $rgbGenDk -variable valGenRD -width $widthScale

checkbutton .fCtrlGen.fRadioSlide.fUpRandDn.cGenRP -background $rgbGen -text "rp rand place"    -variable valGenRP -anchor w
checkbutton .fCtrlGen.fRadioSlide.fUpRandDn.cGenRR -background $rgbGen -text "rr rand rotate"   -variable valGenRR -anchor w

scale       .fCtrlGen.fRadioSlide.fUpRandDn.sGenRS -background $rgbGen -command "setGenRS" -label "rs rand scale" \
                                       -showvalue 1 -from   1 -to 100   -orient horizontal \
                                       -troughcolor $rgbGenDk -variable valGenRS -width $widthScale

pack configure \
   .fCtrlGen.fRadioSlide.fUpRandDn.sGenRD \
   .fCtrlGen.fRadioSlide.fUpRandDn.cGenRP \
   .fCtrlGen.fRadioSlide.fUpRandDn.cGenRR \
   .fCtrlGen.fRadioSlide.fUpRandDn.sGenRS \
   -side top -fill y -anchor "n" -expand 1

pack configure \
   .fCtrlGen.fRadioSlide.fRadioLevel .fCtrlGen.fRadioSlide.fUpRandDn \
   -side left -fill x -anchor "n"
   # -expand 1

scale       .fCtrlGen.sGenD   -background $rgbGen  -command "setGenD" -label "d down to"  \
                              -showvalue 1 -from   0 -to 100   -orient horizontal \
                              -troughcolor $rgbGenDk -variable valGenD -width $widthScale
button      .fCtrlGen.bGen    -background $rgbGenB -command "cmdGen"  -text "Gen"    -width $widthBut
button      .fCtrlGen.bExit   -background $rgbGen -command exit      -text "Exit "   -width $widthBut
label       .fCtrlGen.lGenBlank0  -background $rgbGen -text " "
label       .fCtrlGen.lGenBlank1  -background $rgbGen -text " "

pack configure \
   .fCtrlGen.lGenTgaHfIn   .fCtrlGen.eGenTgaHfIn   \
   .fCtrlGen.lGenPlantMap  .fCtrlGen.eGenPlantMap  \
   .fCtrlGen.lGenObjName   .fCtrlGen.eGenObjName   \
   .fCtrlGen.lGenIncOut    .fCtrlGen.eGenIncOut    \
   .fCtrlGen.lGenBlank0    \
   .fCtrlGen.sGenS         \
   .fCtrlGen.sGenU         \
   .fCtrlGen.fRadioSlide   \
   .fCtrlGen.sGenD         \
   .fCtrlGen.bGen          \
   .fCtrlGen.lGenBlank1    \
   -side top -fill x
   # -expand 1

# --- Catchall frame for Graphic display, display keys, and GEN xyz ---
#
frame    .fMisc -width $widthGIF
canvas   .fMisc.cGraphic -width $widthGIF -height $heightGIF -relief sunken -border 2

# GEN xyz and display keys
#
frame    .fMisc.fCtrl   -background $rgbGen -width $widthGIF

# GEN xyz dimension controls
#
   #
   # -x -y -z dimension ( coordinate at corner opposite <0, 0, 0> )
   #

####################################################
# Get scale data prior to creating sliders & packing
#
# TBD - enable Gen X, Y, and Z dynamically with modalActive = active
# ... al other functions respect modalActive okay, these controls just
# stay in the initial state even if repacked --- why?
#
frame    .fMisc.fCtrl.fGenXYZ -background $rgbGen -borderwidth $spFrame -relief $relFrame -width $widthGIF

scale    .fMisc.fCtrl.fGenXYZ.sGenX -background $rgbGen -command "setGenX"             \
                                    -label "x left-right"                              \
                                    -showvalue 1 -from   1 -to [expr $scaleWidth * 2] -orient horizontal \
                                    -troughcolor $rgbGenDk -variable scaleWidth \
                                    -width $widthScale   \
                                    -length $lengthGenXYZ
                                    # -state $modalActive


scale    .fMisc.fCtrl.fGenXYZ.sGenZ -background $rgbGen  -command "setGenZ"            \
                                    -label "z near-far"                                \
                                    -showvalue 1 -from   1 -to [expr $scaleLength * 2] -orient horizontal \
                                    -troughcolor $rgbGenDk -variable scaleLength \
                                    -width $widthScale   \
                                    -length $lengthGenXYZ
                                    # -state $modalActive

scale    .fMisc.fCtrl.fGenXYZ.sGenY -background $rgbGen  -command "setGenY"            \
                                    -label "y low-high"                                \
                                    -showvalue 1 -from   1 -to [expr $scaleHeight * 2] -orient horizontal  \
                                    -troughcolor $rgbGenDk -variable scaleHeight \
                                    -width $widthScale   \
                                    -length $lengthGenXYZ
                                    # -state $modalActive

# TBD radio buttons for 100max/25max and 1000max/250max for GenX GenZ / GenY
#

# TBD checkboxes to lock Camera and Light(s) to xxxScale
#
frame .fMisc.fCtrl.fGenXYZ.fLock -background $rgbGen -borderwidth $spFrame -relief $relFrame
label       .fMisc.fCtrl.fGenXYZ.fLock.lLock -background $rgbGen     -text "lock"
checkbutton .fMisc.fCtrl.fGenXYZ.fLock.cCameraLock -background $rgbGen -text "camera" -variable cameraLock -anchor w
checkbutton .fMisc.fCtrl.fGenXYZ.fLock.cLightLock  -background $rgbGen -text "light"  -variable lightLock -anchor w

pack configure \
   .fMisc.fCtrl.fGenXYZ.fLock.lLock       \
   .fMisc.fCtrl.fGenXYZ.fLock.cCameraLock \
   .fMisc.fCtrl.fGenXYZ.fLock.cLightLock  \
   -side top -fill y -anchor "w" -expand 1

# GEN xyz
#
# pack this too...    .fMisc.fCtrl.fGenXYZ.fLock \
#
pack configure \
   .fMisc.fCtrl.fGenXYZ.sGenX \
   .fMisc.fCtrl.fGenXYZ.sGenZ \
   .fMisc.fCtrl.fGenXYZ.sGenY \
   -side left -fill x -anchor "w" -expand 1

# GEN xyz and display keys
#
#   .fMisc.fCtrl.fDisplayKeys  \
#
pack configure \
   .fMisc.fCtrl.fGenXYZ       \
   -side left -fill x -anchor "w" -expand 1

   # vertical stack
   # -side top -fill y -anchor "nw"

# Graphic display, with GEN xyz controls and display keys
#
pack configure \
   .fMisc.cGraphic   \
   .fMisc.fCtrl      \
   -side top -fill y -expand 1

# --- ROUGH application ---

# ROUGH application variables
#
variable fileRoughTgaHfIn
variable fileRoughGifMapOut
variable imageRough
variable valRoughR
variable valRoughHA valRoughHB valRoughHC valRoughHD valRoughHE

# ROUGH defaults
#
set fileRoughTgaHfIn    "gfoutput.tga"
set fileRoughGifMapOut  "gttest.gif"
set valRoughHE          46
set valRoughHD          42
set valRoughHC          30
set valRoughHB          25
set valRoughHA           8
set valRoughR            8

# ROUGH application methods
#
proc setRoughR {r} {
   global valRoughR
   set valRoughR $r
   # puts stdout $valRoughR
}
proc setRoughHA {ha} {
   global valRoughHA
   set valRoughHA $ha
   # puts stdout $valRoughHA
}
proc setRoughHB {hb} {
   global valRoughHB
   set valRoughHB $hb
   # puts stdout $valRoughHB
}
proc setRoughHC {hc} {
   global valRoughHC
   set valRoughHC $hc
   # puts stdout $valRoughHC
}
proc setRoughHD {hd} {
   global valRoughHD
   set valRoughHD $hd
   # puts stdout $valRoughHD
}
proc setRoughHE {he} {
   global valRoughHE
   set valRoughHE $he
   # puts stdout $valRoughHE
}

proc cmdRough {} {
   global imageRough
   global xlocGIF ylocGIF
   global valRoughR
   global valRoughHA valRoughHB valRoughHC valRoughHD valRoughHE
   global fileRoughTgaHfIn fileRoughGifMapOut

   # puts stdout "Rough r = $valRoughR, ha = $valRoughHA, hb = $valRoughHB, hc = $valRoughHC, hd = $valRoughHD, he = $valRoughHE."
   # puts stdout "Input = $fileRoughTgaHfIn , Output = $fileRoughGifMapOut"
   # exec ../rough -igfoutput.tga -ogtoutput.gif -r$valRoughR -ha$valRoughHA -hb$valRoughHB -hc$valRoughHC -hd$valRoughHD -he$valRoughHE

   # Don't spawn processes in the output-processing stream!
   # Consumers may starve or eat old data.
   # Rough produces a topo map for susbsequent display.
   #
   exec rough \
         -i$fileRoughTgaHfIn \
         -o$fileRoughGifMapOut \
         -r$valRoughR \
         -ha$valRoughHA \
         -hb$valRoughHB \
         -hc$valRoughHC \
         -hd$valRoughHD \
         -he$valRoughHE
   $imageRough read $fileRoughGifMapOut
   puts stdout "Rough done, executed by GT"
   # wipe, then immediately redisplay topo map so user doesn't see any blink
   #
   .fMisc.cGraphic delete all
   .fMisc.cGraphic create image $xlocGIF $ylocGIF -image $imageRough
   drawCamera
   drawLights

   # pack .fMisc.cGraphic
}

# ROUGH application controls
#
frame    .fCtrlRough                   -background $rgbRough   -height 700 -borderwidth $spFrame -relief $relFrame
label    .fCtrlRough.lRoughTgaHfIn     -background $rgbRough   -text "i Tga heightfield in: "
entry    .fCtrlRough.eRoughTgaHfIn     -background $colorText  -textvariable fileRoughTgaHfIn   -width $widthText
label    .fCtrlRough.lRoughGifMapOut   -background $rgbRough   -text "o Gif plant map out: "
entry    .fCtrlRough.eRoughGifMapOut   -background $colorText  -textvariable fileRoughGifMapOut -width $widthText

scale    .fCtrlRough.sRoughR     -background $rgbRough -command "setRoughR"  -label " r roughness" \
                                 -orient horizontal -showvalue 1 -from   10 -to   1   \
                                 -troughcolor $rgbRoughDk -variable valRoughR -width $widthScale
scale    .fCtrlRough.sRoughHA    -background $rgbRough -command "setRoughHA" -label "ha height a"  \
                                 -orient horizontal -showvalue 1 -from   0 -to 100   \
                                 -troughcolor $rgbRoughDk -variable valRoughHA -width $widthScale
scale    .fCtrlRough.sRoughHB    -background $rgbRough -command "setRoughHB" -label "hb height b"  \
                                 -orient horizontal -showvalue 1 -from   0 -to 100   \
                                 -troughcolor $rgbRoughDk -variable valRoughHB -width $widthScale
scale    .fCtrlRough.sRoughHC    -background $rgbRough -command "setRoughHC" -label "hc height c"  \
                                 -orient horizontal -showvalue 1 -from   0 -to 100   \
                                 -troughcolor $rgbRoughDk -variable valRoughHC -width $widthScale
scale    .fCtrlRough.sRoughHD    -background $rgbRough -command "setRoughHD" -label "hd height d"  \
                                 -orient horizontal -showvalue 1 -from   0 -to 100   \
                                 -troughcolor $rgbRoughDk -variable valRoughHD -width $widthScale
scale    .fCtrlRough.sRoughHE    -background $rgbRough -command "setRoughHE" -label "he height e"  \
                                 -orient horizontal -showvalue 1 -from   0 -to 100   \
                                 -troughcolor $rgbRoughDk -variable valRoughHE -width $widthScale

button   .fCtrlRough.bRough      -background $rgbRoughB -command { cmdRough } -text "Rough"   -width $widthBut
button   .fCtrlRough.bExit       -background $rgbRough -command exit         -text "Exit "    -width $widthBut

label    .fCtrlRough.lBlank0     -background $rgbRough  -text " "
label    .fCtrlRough.lBlank1     -background $rgbRough  -text " "

pack configure \
   .fCtrlRough.lRoughTgaHfIn     \
   .fCtrlRough.eRoughTgaHfIn     \
   .fCtrlRough.lRoughGifMapOut   \
   .fCtrlRough.eRoughGifMapOut   \
   .fCtrlRough.lBlank0           \
   .fCtrlRough.sRoughR           \
   .fCtrlRough.sRoughHE          \
   .fCtrlRough.sRoughHD          \
   .fCtrlRough.sRoughHC          \
   .fCtrlRough.sRoughHB          \
   .fCtrlRough.sRoughHA          \
   .fCtrlRough.bRough            \
   .fCtrlRough.lBlank1           \
   -side top  -fill x -expand 1
   # -anchor "n"


###############################
# --- Application selector ---
#
# Current apps are Gen and Rough
# FileOp interface is done, but not the methods
# Cutter is TBD also.

# Application selector variables used in realtime,
# set to initial application (upon wake-up):
#
# set app      CtrlGen           set app      CtrlRough  #; control frame name
# set colorApp $rgbGen    -OR-   set colorApp $rgbRough  #; app theme color
# set nameApp  Gen               set nameApp  Rough      #; app proper name
#
variable app colorApp nameApp

set app      CtrlFile
set colorApp $rgbFileOp
set nameApp  FileOp

set oldApp        $app
set oldColorApp   $colorApp
set oldNameApp    $nameApp

# These are fixes
# . removed $ from set colorApp rgbFileOp
# . replaced $
# . added global rgbFileOp
#
# modalActive is used to lock Camera and Lights
# control panels open 'til Done is hit,
# also for Gen X, Y, and Z locks.
#
proc setFileOp {} {
   global modalActive
   global app colorApp rgbFileOp

   if { $modalActive == "active" } { } \
   else {
      grid remove .f$app
      set app CtrlFile
      set colorApp $rgbFileOp
      grid .f$app -column 0 -row 0 -sticky "sw"
   }
}
# removed $ from set colorApp rgbGen
# replaced $
# added global rgbGen
proc setGen {} {
   global modalActive
   global app colorApp rgbGen

   if { $modalActive == "active" } { } \
   else {
      if {$app == "CtrlGen"} {cmdGen} \
      else {
         grid remove .f$app
         set app CtrlGen
         set colorApp $rgbGen
         grid .f$app -column 0 -row 0 -sticky "sw"
      }
   }
}
# removed $ from set colorApp rgbRough
# replaced $
# added global rgbRough
proc setRough {} {
   global modalActive
   global app colorApp rgbRough

   if { $modalActive == "active" } { } \
   else {
      if {$app == "CtrlRough"} {cmdRough} \
      else {
         grid remove .f$app
         set app CtrlRough
         set colorApp $rgbRough
         grid .f$app -column 0 -row 0 -sticky "sw"
      }
   }
}
proc setCutter {} {
   global modalActive
   global app colorApp rgbCutter

   if { $modalActive == "active" } { } \
   else {
      if {$app == "CtrlCutter"} {cmdCutter} \
      else {
         grid remove .f$app
         set app CtrlCutter
         set colorApp $rgbCutter
         grid .f$app -column 0 -row 0 -sticky "sw"
      }
   }
}

# Application selector widgets
#
frame    .fAppSelect          -background Red
button   .fAppSelect.bFileOp  -background $rgbFileOp   -command "setFileOp" -text "FileOp"   -width $widthBut
button   .fAppSelect.bRough   -background $rgbRough    -command "setRough"  -text "Rough"    -width $widthBut
button   .fAppSelect.bGen     -background $rgbGen      -command "setGen"    -text "Gen"      -width $widthBut
button   .fAppSelect.bCutter  -background $rgbCutter   -command "setCutter" -text "Cutter"   -width $widthBut

pack     .fAppSelect.bFileOp  \
         .fAppSelect.bRough   \
         .fAppSelect.bGen     \
         .fAppSelect.bCutter  \
         -side left -fill x -anchor "center" -expand 1

################################################
# Help, setCamera, setLight, objectStats buttons

proc setCamera {} {
   global modalActive
   global app        oldApp
   global colorApp   oldColorApp
   global nameApp    oldNameApp
   global rgbCamera

   if { $modalActive == "active" } { } \
   else {
      set oldApp        $app
      set oldColorApp   $colorApp
      set oldNameApp    $nameApp
      initCamera
      grid remove .f$app
      set app CtrlCamera
      set colorApp $rgbCamera
      grid .f$app -column 0 -row 0 -sticky "sw"
   }
}

proc setLight {} {
   global modalActive
   global app        oldApp
   global colorApp   oldColorApp
   global nameApp    oldNameApp
   global rgbLight

   if { $modalActive == "active" } { } \
   else {
      set oldApp        $app
      set oldColorApp   $colorApp
      set oldNameApp    $nameApp
      initLights
      grid remove .f$app
      set app CtrlLight
      set colorApp $rgbLight
      grid .f$app -column 0 -row 0 -sticky "sw"
   }
}

proc cmdHelp {} {
}

proc cmdObjs {} {
}

# Help setCamera, setLight, objectStats buttons
#
frame .fButtonKey -background Yellow
button .fButtonKey.bHelp      -background Black       -command "cmdHelp"   -text "Help"   \
                              -foreground White -width 18
button .fButtonKey.bCamera    -background $rgbCamera  -command "setCamera" -text "Camera" \
                              -foreground White -width 2
button .fButtonKey.bLights    -background $rgbLight   -command "setLight"  -text "Lights" \
                              -foreground White -width 2
button .fButtonKey.bObjects   -background Green       -command "cmdObjs"   -text "Objs"   \
                              -foreground Black  -width 2
button .fButtonKey.bGandE     -background #000099     -command exit        -text "G & E"  \
                              -foreground #ffcc00
                              # -width $widthBut
button .fButtonKey.bExit     -command exit -text "Exit " -width 18

frame    .fButtonKey.fLockCamera -background $rgbCamera
button   .fButtonKey.fLockCamera.bCamera -background $rgbCamera -command "setCamera" \
                           -text " Camera "  -foreground White  \
                           -width 6
                           # $widthButK
                           # -height $heightKeys

checkbutton .fButtonKey.fLockCamera.cCameraLock  -background $rgbCameraLt \
                           -text "lock" -variable lightLock \
                           -height 1
label .fButtonKey.fLockCamera.lCameraBlank0 -text " " -background $rgbCamera
label .fButtonKey.fLockCamera.lCameraBlank1 -text " " -background $rgbCamera

pack configure \
   .fButtonKey.fLockCamera.bCamera        \
   .fButtonKey.fLockCamera.lCameraBlank0  \
   .fButtonKey.fLockCamera.cCameraLock    \
   .fButtonKey.fLockCamera.lCameraBlank1  \
   -side left -fill x -expand 1

frame    .fButtonKey.fLockLights -background $rgbLight
button   .fButtonKey.fLockLights.bLights -background $rgbLight -command "setLight" \
                           -text " Lights "  -foreground White  \
                           -width 6
                           # $widthButK
                           # -height $heightKeys
checkbutton .fButtonKey.fLockLights.cLightLock  -background $rgbLightLt \
                           -text "lock" -variable lightLock \
                           -height 1
label .fButtonKey.fLockLights.lLightBlank0 -text " " -background $rgbLight
label .fButtonKey.fLockLights.lLightBlank1 -text " " -background $rgbLight

pack configure   \
   .fButtonKey.fLockLights.bLights        \
   .fButtonKey.fLockLights.lLightBlank0   \
   .fButtonKey.fLockLights.cLightLock     \
   .fButtonKey.fLockLights.lLightBlank1   \
   -side left -fill x -expand 1

# removed  locks - they look too busy here
#      .fButtonKey.fLockCamera \
#      .fButtonKey.fLockLights \
#
pack configure   \
      .fButtonKey.bLights     \
      .fButtonKey.bObjects    \
      .fButtonKey.bCamera     \
      .fButtonKey.bExit       \
      .fButtonKey.bHelp       \
      -side left -fill x -anchor "center" -expand 1

####################################################################
# Initialize GUI and fire-up
#

# grid for all, makes all frames display (now)
#
grid  .f$app            -column 0 -row 0 -rowspan 1 -sticky "sw"
grid  .fAppSelect       -column 0 -row 1 -rowspan 1 -sticky "we"
grid  .fMisc            -column 1 -row 0 -rowspan 1 -sticky "sw"
# grid  .bHelp            -column 1 -row 1 -rowspan 1 -sticky "s"
grid  .fButtonKey       -column 1 -row 1 -rowspan 1 -sticky "we"

# Immediate command to display window color?
#

# Immediate camera initialization (display masked 'til Camera selected)
#
set xLo [expr              (( $widthGIFf  / $scaleWidth  ) * $cameraX)]
set zLo [expr $heightGIF - (( $heightGIFf / $scaleLength ) * $cameraZ)]
set xHi [expr              (( $widthGIFf  / $scaleWidth  ) * $cameraLookX)]
set zHi [expr $heightGIF - (( $heightGIFf / $scaleLength ) * $cameraLookZ)]

set oldCamera [ .fMisc.cGraphic create line $xLo $zLo $xHi $zHi -arrow last -fill Red -width $widthLine]

# Immediate light initialization (display masked 'til Light selected)
#
set xLo [expr              (( $widthGIFf  / $scaleWidth  ) * $light1x) - $radiusLight]
set zLo [expr $heightGIF - (( $heightGIFf / $scaleLength ) * $light1z) - $radiusLight]
set xHi [expr              (( $widthGIFf  / $scaleWidth  ) * $light1x) + $radiusLight]
set zHi [expr $heightGIF - (( $heightGIFf / $scaleLength ) * $light1z) + $radiusLight]

set oldLight1 [ .fMisc.cGraphic create oval $xLo $zLo $xHi $zHi -fill Blue ]

set xLo [expr              (( $widthGIFf  / $scaleWidth  ) * $light2x) - $radiusLight]
set zLo [expr $heightGIF - (( $heightGIFf / $scaleLength ) * $light2z) - $radiusLight]
set xHi [expr              (( $widthGIFf  / $scaleWidth  ) * $light2x) + $radiusLight]
set zHi [expr $heightGIF - (( $heightGIFf / $scaleLength ) * $light2z) + $radiusLight]

set oldLight2 [ .fMisc.cGraphic create oval $xLo $zLo $xHi $zHi -fill Blue ]

set xLo [expr              (( $widthGIFf  / $scaleWidth  ) * $light3x) - $radiusLight]
set zLo [expr $heightGIF - (( $heightGIFf / $scaleLength ) * $light3z) - $radiusLight]
set xHi [expr              (( $widthGIFf  / $scaleWidth  ) * $light3x) + $radiusLight]
set zHi [expr $heightGIF - (( $heightGIFf / $scaleLength ) * $light3z) + $radiusLight]

set oldLight3 [ .fMisc.cGraphic create oval $xLo $zLo $xHi $zHi -fill Blue ]

# Immediate command to display signon message
#
.fMisc.cGraphic create image $xlocGIF $ylocGIF -image $imageRough

# Immediate command to display signon text
#
.fMisc.cGraphic create text 220 60 -justify left -font {Helvetica -24 bold italic } -fill White \
   -text \
   "Genesis Toolkit is \x00A9 1998-2002 \n\
    http://www.schrammel.org/"
.fMisc.cGraphic create text 205 100 -justify left -font {Helvetica -18 bold italic } -fill White \
   -text \
    "Object placement for POV-Ray terrains"
.fMisc.cGraphic create text 210 180 -justify left -font {Helvetica -24 bold italic } -fill White \
   -text \
   "GT is \x00A9 2002 \n\
    http://www.bowlvisions.com/"
.fMisc.cGraphic create text 220 220 -justify left -font {Helvetica -18 bold italic } -fill White \
   -text \
    "GUI for Genesis, POV-Ray lights & camera"

# Immediate command to display initial application tab's control panel,
# the variable nameApp is set above in the Application selector code.
#
# the command set$nameApp is NOT a misprint
#
set$nameApp

# end GT application



# --- tutorial snip on tk_messagebox
#
# proc doIt {label} {
# set button \
# [tk_messageBox \
# -icon question \
# -type yesno \
# -title Message \
# -parent . \
# -message "Do you like me so far?"]
# $label configure -text $button
# }
#
# label .l -text "I'm not sure yet"
# button .b -text "Do you like me?" \
# -command "doIt .l"
#
# grid .b -row 0 -column 0
# grid .l -row 0 -column 1
#
# --- end tutorial





